import File from './utilities/File'
import Blob from './utilities/Blob'

type DataUriToBlob = (dataUri: string, name?: string) => Blob

const DATA_URI_PREFIX = 'data:'
const DATA_TYPE_DELIMITER = ';'
const DATA_START = ','

const dataUriToBlob: DataUriToBlob = (dataUri, name?) => {
  if ((typeof dataUri as unknown) !== 'string') {
    throw TypeError('Argument should be a string.')
  }

  const [encoding, base64] = dataUri.slice(DATA_URI_PREFIX.length).split(DATA_START)
  const binary = atob(base64)
  const [type] = encoding.split(DATA_TYPE_DELIMITER)
  const ab = new ArrayBuffer(binary.length)
  const ia = new Uint8Array(ab)
  for (let i = 0; i < binary.length; i++) {
    ia[i] = binary.charCodeAt(i)
  }
  if (typeof name! === 'string') {
    return new File([ab], name, { type })
  }
  return new Blob([ab], { type })
}

export default dataUriToBlob
